#!/usr/bin/perl -w
#
# Sensor for gathering PosgreSQL performance data for Zabbix
# Copyright (c) 2008 Jakov Sosic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Changes and Modifications
# * Thu May 15 03:40:39 CEST 2008	
# 	- script created

use strict;
use warnings;
use DBI;

# signal handling
$SIG{'ALRM'} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
};

local $SIG{TERM} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
};

alarm 3;

my $dbhost = $ENV{'dbhost'} || '127.0.0.1';
my $dbname = $ENV{'dbname'} || 'template1';
my $dbuser = $ENV{'dbuser'} || 'zabbix_agentd';
my $dbpass = $ENV{'dbpass'} || 'p4ssw0rdZabbi><';

my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd";

# Database connection
    my $Con = "DBI:Pg:dbname=$dbname;host=$dbhost";
    my $Dbh = DBI->connect ($Con,
                            $dbuser,
			    $dbpass,
			    {RaiseError =>1}) || do { print "0\n"; die "Unable to access Database $dbname on host $dbhost as user $dbuser.\nError returned was: ". $DBI::errstr; };
 
# Locks & exclusive locks
    alarm 3;
    my $sql="SELECT mode,COUNT(mode) FROM pg_locks GROUP BY mode ORDER BY mode;";
    my $sth;
    $sth = $Dbh->prepare($sql) or do { print "0\n"; die($Dbh->errstr); };
    $sth->execute() or do { print "0\n"; die($Dbh->errstr); };
    my $locks = 0;
    my $exlocks = 0;
    while (my ($mode, $count) = $sth->fetchrow ()) {
	if ($mode =~ /exclusive/i) {
	    $exlocks = $exlocks + $count;
	}
	$locks = $locks+$count;
    }
    #print "postgresql.locks $locks\n";
    #print "postgresql.exlocks $exlocks\n";

# Number of active processes
    alarm 3;
    $sql="select sum(numbackends) from pg_stat_database";
    $sth = $Dbh->prepare($sql) or do { print "0\n"; die($Dbh->errstr); };
    $sth->execute() or do { print "0\n"; die($Dbh->errstr); };
    my $activeproc = $sth->fetchrow();
    #print "postgresql.activeproc $activeproc\n";

# Commited and rolled-back transactions
   alarm 3;
   $sql="select sum(xact_commit) from pg_stat_database";
   $sth = $Dbh->prepare($sql) or do { print "0\n"; die($Dbh->errstr); };
   $sth->execute() or do { print "0\n"; die($Dbh->errstr); };
   my $commit = $sth->fetchrow();
   #print "postgresql.commit $commit\n";
   alarm 3;
   $sql="select sum(xact_rollback) from pg_stat_database";
   $sth = $Dbh->prepare($sql) or do { print "0\n"; die($Dbh->errstr); };
   $sth->execute() or do { print "0\n"; die($Dbh->errstr); };
   my $rollback = $sth->fetchrow();
   #print "postgresql.rollback $rollback\n";
# Tablespace sizes
   alarm 3;
   my $pg_default=0;
   my $pg_global=0;
   $sql="SELECT spcname, PG_TABLESPACE_SIZE(oid) FROM pg_tablespace ORDER BY 1;";
   $sth = $Dbh->prepare($sql) or do { print "0\n"; die($Dbh->errstr); };
   $sth->execute() or do { print "0\n"; die($Dbh->errstr); };
   alarm 3;
   while (my ($mode, $count) = $sth->fetchrow ()) {
      if ($mode =~ /default/i) {
	 $pg_default=$count;
      } elsif ($mode =~ /global/i) {
	 $pg_global=$count;
      }
   }
   #print "postgresql.pg_default $pg_default\n";
   #print "postgresql.pg_global $pg_global\n";

# Number of processes
   alarm 3;
   my $numprocs = 0;
   open(PS_F, "ps -U postgres|grep -v TTY|");
   while (<PS_F>) {
      $numprocs = $numprocs + 1;
   }
   close(PS_F); 
   #my $procnum = system("ps -U postgres | grep -v TTY | wc -l");

# send everything, if sending fails exit with zero
alarm 5;
if ( qx($zabbix_send_command -k postgresql.locks -o $locks) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k postgresql.exlocks -o $exlocks) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k postgresql.activeproc -o $activeproc) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k postgresql.commit -o $commit) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k postgresql.rollback -o $rollback) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k postgresql.pg_default -o $pg_default) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k postgresql.pg_global -o $pg_global) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k postgresql.numprocs -o $numprocs) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
alarm 0;
print("1\n");

#eof
